/**
*	lcd.c
*	LCDhCoi20~4sΉC^[tF[X4rbgf[^j
*	PORT 3		b0- D4
*			b1- D5
*			b2- D6
*			b3- D7
*	PORT 7		b4- E
*			b5- R/~W
*			b6- RS
*	Copyright (c)Junnichi Tomaru 2006
*/
#include	<machine.h>
#include	"iodefine.h"
#include	"lcd.h"
#include	"userdef.h"

extern	void	waitms( word time );

#define	WAIT_BUSY()	while(lcdbusy());

#define	LCD_DATA	IO.PDR3.BYTE

#define ASSERT_RS( )	{ IO.PDR8.BIT.B5 = 1; }
#define ASSERT_RW( )	{ IO.PDR8.BIT.B6 = 1; }
#define ASSERT_E( )	{ IO.PDR8.BIT.B7 = 1; }

#define NEGATE_RS( )	{ IO.PDR8.BIT.B5 = 0; }
#define NEGATE_RW( )	{ IO.PDR8.BIT.B6 = 0; }
#define NEGATE_E( )	{ IO.PDR8.BIT.B7 = 0; }

#define WRITE_MODE( )	{ IO.PCR3 |= 0x0f; }
#define READ_MODE( )	{ IO.PCR3 &= 0xf0; }


/**
*	f[^Zbg
*	 in	f[^
*	out	Ȃ
*/
void	write_data( byte data )
{
	byte	dat;

	dat = LCD_DATA & 0xf0;
	dat |= data & 0x0f;
	LCD_DATA = dat;
}


/**
*	kbcɃf[^
*	 in	f[^
*	out	Ȃ
*/
void	lcdwrite( unsigned char data )
{
	WRITE_MODE( );

	NEGATE_RW( );
	write_data( data >> 4 );/* tAS:40ns min. */
	ASSERT_E( );
	nop( );		/* tWEH: 230ns min. */
	nop( );
	nop( );
	nop( );
	NEGATE_E( );
	write_data( data );
	ASSERT_E( );
	nop( );		/* tWEH: 230ns min. */
	nop( );
	nop( );
	nop( );
	NEGATE_E( );
}


/**
*	kbcf[^ǂ
*	 in	Ȃ
*	out	f[^
*/
unsigned char	lcdread( void )
{
	byte	data;

	READ_MODE( );

	ASSERT_RW( );
	nop( );		/* tAS:40ns min. */
	ASSERT_E( );
	nop( );		/* tDDR:160ns max. */
	nop( );
	nop( );
	data = LCD_DATA << 4;
	NEGATE_E( );
	nop( );
	nop( );
	ASSERT_E( );	/* tcycE:500ns min. */
	nop( );		/* tDDR:160ns max. */
	nop( );
	nop( );
	data |= LCD_DATA & 0x0f;
	NEGATE_E( );

	return data;
}


/**
*	CXgNV
*	 in	f[^
*	out	Ȃ
*/
void	lcdwriteir( unsigned char data )
{
	NEGATE_RS( );
	lcdwrite( data );
}


/**
*	CXgNVǂ
*	 in	Ȃ
*	out	f[^
*/
unsigned char	lcdreadir( void )
{
	NEGATE_RS( );
	return	lcdread( );
}


/**
*	f[^WX^
*	 in	f[^
*	out	Ȃ
*/
void	lcdwritedr( unsigned char data )
{
	ASSERT_RS( );
	lcdwrite( data );
}


/**
*	f[^WX^ǂ
*	 in	Ȃ
*	out	f[^
*/
unsigned char	lcdreaddr( void )
{
	ASSERT_RS( );
	return	lcdread( );
}


/**
*	CXgNViSrbgj
*	 in	f[^
*	out	Ȃ
*/
void	lcdwriteir4( unsigned char data )
{
	WRITE_MODE( );

	NEGATE_RS( );
	NEGATE_RW( );
	write_data( data );
	nop( );
	nop( );
	ASSERT_E( );
	nop( );
	nop( );
	nop( );
	nop( );
	NEGATE_E( );
}


/**
*	rW[Ԏ
*	 in	Ȃ
*	out	0x80:rW[ 0:rW[ł͂Ȃ
*/
char	lcdbusy( void )
{
	return	lcdreadir( ) & 0x80;
}


/**
*	kbc
*	 in	Ȃ
*	out	Ȃ
*/
void	lcdinit( void )
{
	waitms( 30 );
	lcdwriteir4( 0x03 );
	waitms( 5 );
	lcdwriteir4( 0x03 );
	waitms( 1 );
	lcdwriteir4( 0x03 );
	waitms( 1 );
	lcdwriteir4( 0x02 );	/* Data length 4bits */
	waitms( 1 );
	WAIT_BUSY( );
	lcdwriteir( 0x28 );	/* Set function */
	WAIT_BUSY( );
	lcdwriteir( 0x08 );	/* Display off, cursor off, blink off */
	WAIT_BUSY( );
	lcdwriteir( 0x01 );	/* Clear display */
	WAIT_BUSY( );
	lcdwriteir( 0x06 );	/* Entry mode : cursor increment */
	WAIT_BUSY( );
	lcdwriteir( 0x0C );	/* Display on, cursor off, blink off */
}


/**
*	J[\ʒuw
*	 in	x:ʒu
*		y:sʒu
*	out	Ȃ
*/
void	lcdlocate( char x, char y )
{
	char	addr;

	switch ( y ){
		case 0:	addr = 0x00 + x;
			break;
		case 1:	addr = 0x40 + x;
			break;
		case 2:	addr = 0x14 + x;
			break;
		case 3:	addr = 0x54 + x;
			break;
		default:
			return;
	}
	WAIT_BUSY( );
	lcdwriteir( 0x80 | addr );		/* Set DD RAM address */
}


/**
*	ʃNA
*	 in	Ȃ
*	out	Ȃ
*/
void	lcdcls( void )
{
	WAIT_BUSY( );
	lcdwriteir( 0x01 );
}


/**
*	J[\\E\
*	 in	0:\ ̑:\
*	out	Ȃ
*/
void	lcddispcursor( char sw )
{
	WAIT_BUSY( );
	lcdwriteir( 0xc | ( sw ? 0x02 : 0 ) );
}


/**
*	ꕶ\
*	 in	f[^
*	out	Ȃ
*/
void	lcdputch( char c )
{
	WAIT_BUSY( );
	lcdwritedr( c );
}


/**
*	wʒuɈꕶ\iLN^R[hwj
*	 in	   x:ʒu
*		   y:sʒu
*		code:f[^
*	out	Ȃ
*/
void	lcdputcode( char x, char y, char code )
{
	lcdlocate( x, y );
	WAIT_BUSY( );
	lcdwritedr( code );
}


/**
*	\
*	 in	f[^
*	out	Ȃ
*/
void	lcdputstr( char *str )
{
	while( *str != '\0' ){
		lcdputch( *( str++ ) );
	}
}


/**
*	ʒuw肵ĕ\
*	 in	  x:ʒu
*		  y:sʒu
*		str:f[^
*	out	Ȃ
*/
void	lcdprint( char x, char y, char *str )
{
	lcdlocate( x, y );
	lcdputstr( str );
}


/**
*	wʒuw萔̕
*	 in	  x:ʒu
*		  y:sʒu
*		cnt::
*/
void	lcdspace( char x, char y, char cnt )
{
	char	i;

	lcdlocate( x, y );
	for( i = 0; i < cnt; i++ ){
		lcdputch( ' ' );
	}
}


/**
*	sNA
*	 in	NAs
*	out	Ȃ
*/
void	lcdlcls( char line )
{
	lcdspace( 0, line, 16 );
}


/* end of lcd.c */
